/*
 * GPIO Test
 *
 * This Program is CG-LBSTD2PU (ADM5120) Only.
 *  CG-LBSTD2PU          Test Board
 * CN8 1(LED[0]2) ---->
 * CN8 2(LED[1]2) ---->
 * CN8 3(LED[3]1) ---->
 * CN8 4(LED[3]2) ---->
 * CN8 5(GPIO4)   ---->
 * CN8 6
 * CN8 7(GND)     ---->
 * CN8 8(+3.3V)   ---->
 *
 * This Source Program is GPL
 * Created By T.Yoshida at 2008.09.17
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/select.h>
#include <ctype.h>

/* ADM5120 SWITCH Register Base Address */
#define ADM5120_SWITCH_BASE	0x12000000
#define ADM5120_SCR_SIZE	0x1000
#define GPIO_CONF0		0xb8
#define PORT0_LED		0x100
#define PORT1_LED		0x104
#define PORT3_LED		0x10c
#define LED0_LED_IN		(0x00)
#define LED0_LED_LO		(0x02)
#define LED0_LED_HI		(0x03)
#define LED0_LED_MASK		(0x0F)

#define LED1_LED_IN		(LED0_LED_IN<<4)
#define LED1_LED_LO		(LED0_LED_LO<<4)
#define LED1_LED_HI		(LED0_LED_HI<<4)
#define LED1_LED_MASK		(LED0_LED_MASK<<4)

#define LED2_LED_IN		(LED0_LED_IN<<8)
#define LED2_LED_LO		(LED0_LED_LO<<8)
#define LED2_LED_HI		(LED0_LED_HI<<8)
#define LED2_LED_MASK		(LED0_LED_MASK<<8)

#define GPIO_HI	(0x01)
#define GPIO_LO	(0x00)
#define GPIO4_HI	(GPIO_HI<<28)
#define GPIO4_LO	(GPIO_LO<<28)

static void *MapPointer = (void *) -1;
static int MapFD = -1;

static unsigned int PortGpiosave;	/* GPIO Save */
static unsigned int Port0Ledsave;	/* LED0 Port Save */
static unsigned int Port1Ledsave;	/* LED1 Port Save */
static unsigned int Port3Ledsave;	/* LED3 Port Sace */
static int PortGpioFlag = 0;

int InitGPIO(void);
int TermGPIO(void);
int Port0_Led2(int, int);
int Port1_Led2(int, int);
int Port3_Led1(int, int);
int Port3_Led2(int, int);
int Gpio4(int, int);

/*
 * Initialize GPIO Connetor CN8
 */
int InitGPIO(void)
{
    unsigned int pattern;
    if (MapFD != -1)
	close(MapFD);

    MapFD = open("/dev/mem", O_RDWR);
    if (MapFD == -1) {
	perror("/dev/mem");
	return (-1);
    }

    MapPointer = mmap(NULL, ADM5120_SCR_SIZE, PROT_READ | PROT_WRITE,
		      MAP_SHARED, MapFD, ADM5120_SWITCH_BASE);

    if (MapPointer == (void *) -1) {
	perror("mmap");
	return (-1);
    }

    pattern = *(unsigned int *) (MapPointer + GPIO_CONF0);

    *(unsigned int *) (MapPointer + GPIO_CONF0) &= 0xFFFF00FF;
    PortGpiosave = *(unsigned int *) (MapPointer + GPIO_CONF0);
    Port0Ledsave = *(unsigned int *) (MapPointer + PORT0_LED);
    Port1Ledsave = *(unsigned int *) (MapPointer + PORT1_LED);
    Port3Ledsave = *(unsigned int *) (MapPointer + PORT3_LED);
    PortGpioFlag = 1;

    return (0);
}

/*
 * Terminate GPIO CN8
 */
int TermGPIO(void)
{
	/*
    if ((MapPointer) != (void *) -1) {
	if (PortGpioFlag) {
	    *(unsigned int *) (MapPointer + GPIO_CONF0) = PortGpiosave;
	    *(unsigned int *) (MapPointer + PORT0_LED) = Port0Ledsave;
	    *(unsigned int *) (MapPointer + PORT1_LED) = Port1Ledsave;
	    *(unsigned int *) (MapPointer + PORT3_LED) = Port3Ledsave;
	}

    }
    */
    if (MapFD != -1)
	close(MapFD);

    MapFD = -1;
    MapPointer = (void *) -1;
    return (0);
}

void alloff(void)
{
    unsigned int pattern;
    /* LED[0]2 Off */
    pattern = *(unsigned int *) (MapPointer + PORT0_LED) & ~LED2_LED_MASK;
    *(unsigned int *) (MapPointer + PORT0_LED) = pattern | 0x0300;
    /* LED[1]2 Off */
    pattern = *(unsigned int *) (MapPointer + PORT1_LED) & ~LED2_LED_MASK;
    *(unsigned int *) (MapPointer + PORT1_LED) = pattern | 0x0300;
    /* LED[3]1 Off */
    pattern = *(unsigned int *) (MapPointer + PORT3_LED) & ~LED1_LED_MASK;
    *(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0030;
    pattern = *(unsigned int *) (MapPointer + PORT3_LED) & ~LED2_LED_MASK;
    *(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0300;
    /* GPIO4 Off */
    pattern = *(unsigned int *) (MapPointer + GPIO_CONF0) & 0xEFFF00FF;
    *(unsigned int *) (MapPointer + GPIO_CONF0) = pattern | 0x00100000;
}
/*
 * CN8 1 LED[0]2 On and Off
 * Input Value : int : count : Loop Count,Default 10
 *             : int : timer : Wait Timer , Default 3
 */
int Port0_Led2(int count, int timer)
{
    unsigned int pattern;
    int i = 0;

    pattern = *(unsigned int *) (MapPointer + PORT0_LED) & ~LED2_LED_MASK;

    fprintf(stdout,"CN8-1 <Port0 LED2> Check!! ");
    fflush(stdout);
    *(unsigned int *) (MapPointer + PORT0_LED) = pattern | 0x0300;
    for (i = 0; i < count; i++) {
	    fprintf(stdout,"%d ",i);
	    fflush(stdout);
	*(unsigned int *) (MapPointer + PORT0_LED) = pattern | 0x0200;
	sleep(timer);
	*(unsigned int *) (MapPointer + PORT0_LED) = pattern | 0x0300;
	sleep(timer);

    }
    fprintf(stdout,"\n");
    fflush(stdout);
    return (0);

}

/*
 * CN8 2 LED[1]2 On and Off
 * Input Value : int : count : Loop Count,Default 10
 *             : int : timer : Wait Timer , Default 3
 */
int Port1_Led2(int count, int timer)
{
    unsigned int pattern;
    int i = 0;

    pattern = *(unsigned int *) (MapPointer + PORT1_LED) & ~LED2_LED_MASK;

    fprintf(stdout,"CN8-2 <Port1 LED2> Check!! ");
    fflush(stdout);
    *(unsigned int *) (MapPointer + PORT1_LED) = pattern | 0x0300;
    for (i = 0; i < count; i++) {
	    fprintf(stdout,"%d ",i);
	    fflush(stdout);
	*(unsigned int *) (MapPointer + PORT1_LED) = pattern | 0x0200;
	sleep(timer);
	*(unsigned int *) (MapPointer + PORT1_LED) = pattern | 0x0300;
	sleep(timer);

    }
    fprintf(stdout,"\n");
    fflush(stdout);
    return (0);

}

/*
 * CN8 3 LED[3]1 On and Off
 * Input Value : int : count : Loop Count,Default 10
 *             : int : timer : Wait Timer , Default 3
 */
int Port3_Led1(int count, int timer)
{
    unsigned int pattern;
    int i = 0;

    pattern = *(unsigned int *) (MapPointer + PORT3_LED) & ~LED1_LED_MASK;

    fprintf(stdout,"CN8-3 <Port3 LED1> Check!! ");
    fflush(stdout);
    *(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0030;
    for (i = 0; i < count; i++) {
	    fprintf(stdout,"%d ",i);
	    fflush(stdout);
	*(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0020;
	sleep(timer);
	*(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0030;
	sleep(timer);

    }
    fprintf(stdout,"\n");
    fflush(stdout);
    return (0);

}

/*
 * CN8 4 LED[3]2 On and Off
 * Input Value : int : count : Loop Count,Default 10
 *             : int : timer : Wait Timer , Default 3
 */
int Port3_Led2(int count, int timer)
{
    unsigned int pattern;
    int i = 0;

    pattern = *(unsigned int *) (MapPointer + PORT3_LED) & ~LED2_LED_MASK;

    fprintf(stdout,"CN8-4 <Port3 LED2> Check!! ");
    fflush(stdout);
    *(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0300;
    for (i = 0; i < count; i++) {
	    fprintf(stdout,"%d ",i);
	    fflush(stdout);
	*(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0200;
	sleep(timer);
	*(unsigned int *) (MapPointer + PORT3_LED) = pattern | 0x0300;
	sleep(timer);

    }
    fprintf(stdout,"\n");
    fflush(stdout);
    return (0);

}

/*
 * CN8 5 GPIO4 On and Off
 * Input Value : int : count : Loop Count,Default 10
 *             : int : timer : Wait Timer , Default 3
 */
int Gpio4(int count, int timer)
{
    unsigned int pattern;
    int i = 0;

    pattern = *(unsigned int *) (MapPointer + GPIO_CONF0) & 0xEFFF00FF;

    fprintf(stdout,"CN8-5 <GPIO4> Check!! ");
    fflush(stdout);
    for (i = 0; i < count; i++) {
	    fprintf(stdout,"%d ",i);
	    fflush(stdout);
	*(unsigned int *) (MapPointer + GPIO_CONF0) = pattern | 0x10100000;
	sleep(timer);
	*(unsigned int *) (MapPointer + GPIO_CONF0) = pattern | 0x00100000;
	sleep(timer);

    }
    fprintf(stdout,"\n");
    fflush(stdout);
    return (0);

}

void usage(void)
{
    printf("gpiotest : CG-LBSTD2PU GPIO CN8 Port Test Program\n");
    printf(" LED0 to LED5 On/Off or LED0 to LED5 Flaush\n");
    printf(" gpiotest -[u|U] : Print Usage\n");
    printf("          -w [0-99]: Wait Timer Sec\n");
    printf("                     Default 10 Sec\n");
    printf("          -c [0-99]: Loop Count\n");
    printf("                   : Default 6 Count\n");
    printf("          -p [F]   : Led Flush\n");
    printf("                     Default None:\n");
    printf("\n");
    printf(" Ex1: Wait 5Sec , 10 Loop\n");
    printf("  gpiotest -w 5 -c 10\n");
    printf(" Ex2: Wait 1Sec , 10 Loop , Flush\n");
    printf("  gpiotest -w 1 -c 10 -p F\n");
}

int main(int argc, char *argv[])
{

    int i = 0, waittimer = 3, loopcount = 6;
    char Work[2], LEDPattern[1];
    Work[0] = LEDPattern[0] = '\0';

    if (InitGPIO() != 0) {
	printf("GPIO Connector CN8 Initialize Error...\n");
	exit(-1);
    }
    i=1;
    for(i=1;i<argc;i++){
#ifdef DEBUG
	    printf("argv %d %s\n",i,argv[i]);
#endif
	    if(!strcmp(argv[i],"-w")) {
		    i++;
		    if(argv[i]==NULL) {
			printf("Wait Timer Error..\n");
			usage();
			exit(-1);
		    }
		    waittimer = atoi(argv[i]);
	    } else if(!strcmp(argv[i],"-c")) {
		    i++;
		    if(argv[i]==NULL) {
			printf("Loop Count Error..\n");
			usage();
			exit(-1);
		    }
		    loopcount = atoi(argv[i]);
	    } else if(!strcmp(argv[i],"-p")){
		    i++;
		    if(argv[i]==NULL) {
			printf("LED Pattern Error..\n");
			usage();
			exit(-1);
		    }
		    sprintf(LEDPattern,"%s",argv[i]);
	    } else if((!strcmp(argv[i],"-u"))||(!strcmp(argv[i],"-U"))){
		    usage();
		    exit(1);
	    } else {
		    usage();
		    exit(1);
	    }
    }

    printf("Pattern %s Waittimer %d sec Loop %d\n",LEDPattern,waittimer,loopcount);
    alloff();
    if (!strcmp(LEDPattern, "F") || !strcmp(LEDPattern, "f")) {
	if (loopcount == 0)
	    loopcount = 10;
	for (i = 0; i < loopcount; i++) {
	    Port0_Led2(1, 1);
	    Port1_Led2(1, 1);
	    Port3_Led1(1, 1);
	    Port3_Led2(1, 1);
	    Gpio4(1, 1);
	}
    } else {
	Port0_Led2(loopcount, waittimer);
	Port1_Led2(loopcount, waittimer);
	Port3_Led1(loopcount, waittimer);
	Port3_Led2(loopcount, waittimer);
	Gpio4(loopcount, waittimer);
    }
    TermGPIO();

    exit(0);
}
