/*
 * GPIO LED On,Off,Flash Sample Program
 *
 * This Program is CG-LBSTD2PU (ADM5120) Only.
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/select.h>
#include <ctype.h>
#include "ledtest.h"

/* ADM5120 SWITCH Register Base Address */
#define ADM5120_SWITCH_BASE	0x12000000
#define ADM5120_SCR_SIZE	0x1000

/* ADM5120 GPIO Config Register */
#define GPIO_CONF0		0xb8
#define PORT0_LED		0x100
#define PORT1_LED		0x104
#define PORT2_LED		0x108
#define PORT3_LED		0x10C

#define PORT0_LED_IN		(0x00)
#define PORT0_LED_OUT_FLASH	(0x01)
#define PORT0_LED_OUT_HI	(0x02)
#define PORT0_LED_OUT_LOW	(0x03)
#define PORT0_LED_MASK 		(0x0F)

#define PORT1_LED_IN		(PORT0_LED_IN<<4)
#define PORT1_LED_OUT_FLASH	(PORT0_LED_OUT_FLASH<<4)
#define PORT1_LED_OUT_HI	(PORT0_LED_OUT_HI<<4)
#define PORT1_LED_OUT_LOW	(PORT0_LED_OUT_LOW<<4)
#define PORT1_LED_MASK 		(PORT0_LED_MASK<<4)

#define PORT2_LED_IN		(PORT0_LED_IN<<8)
#define PORT2_LED_OUT_FLASH	(PORT0_LED_OUT_FLASH<<8)
#define PORT2_LED_OUT_HI	(PORT0_LED_OUT_HI<<8)
#define PORT2_LED_OUT_LOW	(PORT0_LED_OUT_LOW<<8)
#define PORT2_LED_MASK 		(PORT0_LED_MASK<<8)

static void *EthMapPointer = (void *) -1;
static int EthMapFD = -1;
static unsigned int Portledsave;

// ----------------------------------------------------------------------------
// Initialized LED
// Ret: 0  = Normal
//      0 != Error
// ----------------------------------------------------------------------------
int InitLEDPort(void)
{

    if (EthMapFD != -1) close(EthMapFD);

    EthMapFD = open("/dev/mem", O_RDWR);
    if (EthMapFD == -1) {
	perror("/dev/mem");
	return (-1);
    }

    EthMapPointer =
	mmap(NULL, ADM5120_SCR_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED,
	     EthMapFD, ADM5120_SWITCH_BASE);
    if (EthMapPointer == (void *) -1) {
	perror("mmap()");
	return (-1);
    }
    Portledsave = *(unsigned int *) (EthMapPointer + PORT2_LED);

    return (0);
}

// ----------------------------------------------------------------------------
// End LED
// Ret: 0  = Normal
//      0 != Error
// ----------------------------------------------------------------------------
int TermLEDPort(void)
{
    if (EthMapPointer != (void *) -1) {
		Portledsave;
    }
    if (EthMapFD != -1) close(EthMapFD);
    EthMapFD = -1;
    EthMapPointer = (void *) -1;
    return (0);
}

int main(int argc, char *argv[])
{
	int i;
    unsigned int pattern;

    InitLEDPort();

    if (EthMapPointer == (void *) -1)
	exit(1);

    pattern =
	*(unsigned int *) (EthMapPointer + PORT2_LED) & ~PORT0_LED_MASK;
    printf("VPN LED Check!!\n");
    for(i=1;i<6;i++) {
		*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern |0x0003;
		sleep(3);
		*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern |0x0002;
		sleep(3);
    }
	*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern |0x0020;
    pattern =
	*(unsigned int *) (EthMapPointer + PORT2_LED) & ~PORT1_LED_MASK;
    printf("STATUS LED Check!!\n");
    for(i=1;i<6;i++) {
		*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern |0x0030;
		sleep(3);
		*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern |0x0020;
		sleep(3);
    }
	*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern |0x0020;
    pattern =
	    *(unsigned int *)(EthMapPointer + PORT2_LED ) & ~PORT2_LED_MASK;
    printf("LED3 Check!!\n");
    for(i=1;i<6;i++) {
    	*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern | 0x0300;
    	sleep(3);
    	*(unsigned int *) (EthMapPointer + PORT2_LED) = pattern | 0x0200;
    	sleep(3);
    } 
    pattern =
	    *(unsigned int *)(EthMapPointer + PORT3_LED ) & ~PORT0_LED_MASK;
    printf("LED4 Check!!\n");
    for(i=1;i<6;i++) {
    	*(unsigned int *) (EthMapPointer + PORT3_LED) = pattern | 0x0002;
    	sleep(3);
    	*(unsigned int *) (EthMapPointer + PORT3_LED) = pattern | 0x0003;
    	sleep(3);
    }
    *(unsigned int *) (EthMapPointer + PORT3_LED) = pattern | 0x0002;

    TermLEDPort();

    exit(0);
}
